Package: MJMbamlss
Type: Package
Title: Multivariate Joint Models with 'bamlss'
Version: 0.1.0
Authors@R: c(person(given = "Nikolaus", family = "Umlauf", role = c("aut"), email = "Nikolaus.Umlauf@uibk.ac.at",
         comment = c(ORCID = "0000-0003-2160-9803")),
	person("Alexander", "Volkmann", email = "alexander.volkmann@hu-berlin.de", role = c("aut", "cre")))
Description: Multivariate joint models of longitudinal and time-to-event data based on functional principal components implemented with 'bamlss'. Implementation for Volkmann, Umlauf, Greven (2023) <arXiv:2311.06409>.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
Depends: R (>= 3.5), mgcv, bamlss
LinkingTo: Rcpp, RcppEigen
Imports: stats, funData, statmod, mvtnorm, zoo, coda, gamm4, Matrix,
        refund, utils, fdapace, sparseFLMM, MFPCA, foreach
Suggests: testthat (>= 3.0.0), splines, tidyverse
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2023-11-27 15:29:07 UTC; alex
Author: Nikolaus Umlauf [aut] (<https://orcid.org/0000-0003-2160-9803>),
  Alexander Volkmann [aut, cre]
Maintainer: Alexander Volkmann <alexander.volkmann@hu-berlin.de>
Repository: CRAN
Date/Publication: 2023-11-27 17:30:05 UTC
Built: R 4.2.3; aarch64-apple-darwin20; 2023-11-27 18:32:07 UTC; unix
Archs: MJMbamlss.so.dSYM
