# Generated by roxygen2: do not edit by hand

S3method(Predict.matrix,unc_pcre.random.effect)
S3method(mgcv::smooth.construct,unc_pcre.smooth.spec)
export(MFPCA_cov)
export(attach_wfpc)
export(mjm_bamlss)
export(preproc_MFPCA)
export(simMultiJM)
import(MFPCA)
import(bamlss)
import(coda)
import(foreach)
import(sparseFLMM)
import(stats)
import(utils)
importFrom(Matrix,Matrix)
importFrom(Matrix,as.matrix)
importFrom(Matrix,bdiag)
importFrom(Matrix,nearPD)
importFrom(Matrix,t)
importFrom(fdapace,FPCA)
importFrom(funData,X)
importFrom(funData,argvals)
importFrom(gamm4,gamm4)
importFrom(mgcv,Predict.matrix)
importFrom(mgcv,PredictMat)
importFrom(mgcv,bam)
importFrom(mgcv,gam)
importFrom(mgcv,predict.gam)
importFrom(mgcv,tensor.prod.model.matrix)
importFrom(refund,fpca.sc)
importFrom(stats,as.formula)
importFrom(stats,model.matrix)
useDynLib(MJMbamlss)
